package ch.dieseite.sqlite3.jdbc;

import java.sql.Array;
import java.sql.Blob;
import java.sql.CallableStatement;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.NClob;
import java.sql.PreparedStatement;
import java.sql.SQLClientInfoException;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.SQLXML;
import java.sql.Savepoint;
import java.sql.Statement;
import java.sql.Struct;
import java.util.Map;
import java.util.Properties;

import ch.dieseite.sqlite3.wrapper.SmallSQLiteWrapper;

public class ConnectionImpl implements Connection{
	private boolean debug = true; //provoziere exceptions was nicht implementiert ist
	
	private SmallSQLiteWrapper wrapper;
	private boolean autocommit = true; //true = transaction-Mgnt is off
	//Autocommit mode is disabled by a BEGIN statement. 
	//Autocommit mode is re-enabled by a COMMIT or ROLLBACK.		

	
	public ConnectionImpl(String filename, SmallSQLiteWrapper wrap) throws SQLException
	{
		wrapper = wrap;
		wrapper.dBOpen(filename, SmallSQLiteWrapper.SQLITE_OPEN_READWRITE);
	}
	
	@Override
	public void setAutoCommit(boolean arg0) throws SQLException {
        if (autocommit == arg0) return;
		autocommit = arg0;
		if (autocommit)
			wrapper.execSQL("COMMIT;");
		else
			wrapper.execSQL("BEGIN;");
	}
	
	@Override
	public void close() throws SQLException {
		wrapper.dBClose();
		wrapper = null;
	}
	
	@Override
	public Statement createStatement() throws SQLException {
		StatementImpl st = new StatementImpl(wrapper);
		return st;
	}
	
	@Override
	public void commit() throws SQLException {
        if (! autocommit)
        {
        	wrapper.execSQL("COMMIT;");
        	wrapper.execSQL("BEGIN;");
        }
				
	}
	@Override
	public void rollback() throws SQLException {
        if (! autocommit)
        {
        	wrapper.execSQL("ROLLBACK;");
        	wrapper.execSQL("BEGIN;");       	
        }
	}
	
	@Override
	public PreparedStatement prepareStatement(String arg0) throws SQLException {
		return new PreparedStatementImpl(wrapper, arg0);
	}

	//unimplemented methods


	@Override
	public boolean isWrapperFor(Class<?> arg0) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return false;
	}

	@Override
	public <T> T unwrap(Class<T> arg0) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return null;
	}

	@Override
	public void clearWarnings() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		
	}

	@Override
	public Array createArrayOf(String arg0, Object[] arg1) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return null;
	}

	@Override
	public Blob createBlob() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return null;
	}

	@Override
	public Clob createClob() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return null;
	}

	@Override
	public NClob createNClob() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return null;
	}

	@Override
	public SQLXML createSQLXML() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return null;
	}

	@Override
	public Statement createStatement(int arg0, int arg1) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return null;
	}

	@Override
	public Statement createStatement(int arg0, int arg1, int arg2)
			throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return null;
	}

	@Override
	public Struct createStruct(String arg0, Object[] arg1) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return null;
	}

	@Override
	public boolean getAutoCommit() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return false;
	}

	@Override
	public String getCatalog() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return null;
	}

	@Override
	public Properties getClientInfo() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return null;
	}

	@Override
	public String getClientInfo(String arg0) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return null;
	}

	@Override
	public int getHoldability() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return 0;
	}

	@Override
	public DatabaseMetaData getMetaData() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return null;
	}

	@Override
	public int getTransactionIsolation() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return 0;
	}

	@Override
	public Map<String, Class<?>> getTypeMap() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return null;
	}

	@Override
	public SQLWarning getWarnings() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return null;
	}

	@Override
	public boolean isClosed() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return false;
	}

	@Override
	public boolean isReadOnly() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return false;
	}

	@Override
	public boolean isValid(int arg0) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return false;
	}

	@Override
	public String nativeSQL(String arg0) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return null;
	}

	@Override
	public CallableStatement prepareCall(String arg0) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return null;
	}

	@Override
	public CallableStatement prepareCall(String arg0, int arg1, int arg2)
			throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return null;
	}

	@Override
	public CallableStatement prepareCall(String arg0, int arg1, int arg2,
			int arg3) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return null;
	}

	@Override
	public PreparedStatement prepareStatement(String arg0, int arg1)
			throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return null;
	}

	@Override
	public PreparedStatement prepareStatement(String arg0, int[] arg1)
			throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return null;
	}

	@Override
	public PreparedStatement prepareStatement(String arg0, String[] arg1)
			throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return null;
	}

	@Override
	public PreparedStatement prepareStatement(String arg0, int arg1, int arg2)
			throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return null;
	}

	@Override
	public PreparedStatement prepareStatement(String arg0, int arg1, int arg2,
			int arg3) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return null;
	}

	@Override
	public void releaseSavepoint(Savepoint arg0) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		
	}

	@Override
	public void rollback(Savepoint arg0) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		
	}


	@Override
	public void setCatalog(String arg0) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		
	}

	@Override
	public void setClientInfo(Properties arg0) throws SQLClientInfoException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		
	}

	@Override
	public void setClientInfo(String arg0, String arg1)
			throws SQLClientInfoException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		
	}

	@Override
	public void setHoldability(int arg0) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		
	}

	@Override
	public void setReadOnly(boolean arg0) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		
	}

	@Override
	public Savepoint setSavepoint() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return null;
	}

	@Override
	public Savepoint setSavepoint(String arg0) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		return null;
	}

	@Override
	public void setTransactionIsolation(int arg0) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		
	}

	@Override
	public void setTypeMap(Map<String, Class<?>> arg0) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in Connection not Implemented");
		
	}

}
