package ch.dieseite.sqlite3.jdbc;

import java.io.InputStream;
import java.io.Reader;
import java.math.BigDecimal;
import java.net.URL;
import java.sql.Array;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.Date;
import java.sql.NClob;
import java.sql.ParameterMetaData;
import java.sql.PreparedStatement;
import java.sql.Ref;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.RowId;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.SQLXML;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.Calendar;

import ch.dieseite.sqlite3.wrapper.SmallSQLiteWrapper;

public class PreparedStatementImpl implements PreparedStatement {
	private boolean debug = true; //provoziere exceptions was nicht implementiert ist
	
	private SmallSQLiteWrapper wrap;
	private String[] prepStmt;
	private String[] values;
	private String batch;
	
	public PreparedStatementImpl(SmallSQLiteWrapper w, String prepStmt)
	{
		this.prepStmt = prepStmt.split("\\?"); //if nothing to split then array.lengh is 1
		values = new String[this.prepStmt.length];
		wrap = w;
	}
	
	@Override
	public void setDouble(int parameterIndex, double x) throws SQLException {
		values[parameterIndex] = ""+x;
	}
		
	@Override
	public void setInt(int parameterIndex, int x) throws SQLException {
		setLong(parameterIndex, x);
	}
	@Override
	public void setLong(int parameterIndex, long x) throws SQLException {
		values[parameterIndex] = ""+x;
	}
	@Override
	public void setBoolean(int parameterIndex, boolean x) throws SQLException {
		values[parameterIndex] = (x==true)? ""+1:""+0;
	}
	
	@Override
	public void setString(int parameterIndex, String x) throws SQLException {	
		//bugfix
		if (x==null) 
			values[parameterIndex] = null;
		else
			values[parameterIndex] = "'"+x+"'";
	}

	//merge all by "?" predefined gaps from SQL-String with values, defined by setString, setBoolean a.s.o.
	@Override
	public void addBatch() throws SQLException {
		batch = "";
		for (int i = 0; i< prepStmt.length; i++)
		{
			String v = (values[i]== null)? "?": values[i];
			batch += (i==0)?prepStmt[i]:v+prepStmt[i];
		}
	}
	
	@Override
	public int[] executeBatch() throws SQLException {
		wrap.execSQL(batch);
		return null;
	}
	
	@Override
	public void close() throws SQLException {
		wrap = null;
		prepStmt = null;
		values = null;
		batch = null;
	}
	
	//not implemented methods
	
	@Override
	public void cancel() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void clearBatch() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void clearWarnings() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public boolean execute(String arg0) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return false;
	}

	@Override
	public boolean execute(String arg0, int arg1) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return false;
	}

	@Override
	public boolean execute(String arg0, int[] arg1) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return false;
	}

	@Override
	public boolean execute(String arg0, String[] arg1) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return false;
	}

	@Override
	public ResultSet executeQuery(String arg0) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return null;
	}

	@Override
	public int executeUpdate(String arg0) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return 0;
	}

	@Override
	public int executeUpdate(String arg0, int arg1) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return 0;
	}

	@Override
	public int executeUpdate(String arg0, int[] arg1) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return 0;
	}

	@Override
	public int executeUpdate(String arg0, String[] arg1) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return 0;
	}

	@Override
	public Connection getConnection() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return null;
	}

	@Override
	public int getFetchDirection() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return 0;
	}

	@Override
	public int getFetchSize() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return 0;
	}

	@Override
	public ResultSet getGeneratedKeys() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return null;
	}

	@Override
	public int getMaxFieldSize() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return 0;
	}

	@Override
	public int getMaxRows() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return 0;
	}

	@Override
	public boolean getMoreResults() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return false;
	}

	@Override
	public boolean getMoreResults(int arg0) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return false;
	}

	@Override
	public int getQueryTimeout() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return 0;
	}

	@Override
	public ResultSet getResultSet() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return null;
	}

	@Override
	public int getResultSetConcurrency() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return 0;
	}

	@Override
	public int getResultSetHoldability() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return 0;
	}

	@Override
	public int getResultSetType() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return 0;
	}

	@Override
	public int getUpdateCount() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return 0;
	}

	@Override
	public SQLWarning getWarnings() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return null;
	}

	@Override
	public boolean isClosed() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return false;
	}

	@Override
	public boolean isPoolable() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return false;
	}

	@Override
	public void setCursorName(String arg0) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setEscapeProcessing(boolean arg0) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setFetchDirection(int arg0) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setFetchSize(int arg0) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setMaxFieldSize(int arg0) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setMaxRows(int arg0) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setPoolable(boolean arg0) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setQueryTimeout(int arg0) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public boolean isWrapperFor(Class<?> arg0) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return false;
	}

	@Override
	public <T> T unwrap(Class<T> arg0) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return null;
	}

	@Override
	public void addBatch(String arg0) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void clearParameters() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public boolean execute() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return false;
	}

	@Override
	public ResultSet executeQuery() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return null;
	}

	@Override
	public int executeUpdate() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return 0;
	}

	@Override
	public ResultSetMetaData getMetaData() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return null;
	}

	@Override
	public ParameterMetaData getParameterMetaData() throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");
		return null;
	}

	@Override
	public void setArray(int parameterIndex, Array x) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setAsciiStream(int parameterIndex, InputStream x)
			throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setAsciiStream(int parameterIndex, InputStream x, int length)
			throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setAsciiStream(int parameterIndex, InputStream x, long length)
			throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setBigDecimal(int parameterIndex, BigDecimal x)
			throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setBinaryStream(int parameterIndex, InputStream x)
			throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setBinaryStream(int parameterIndex, InputStream x, int length)
			throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setBinaryStream(int parameterIndex, InputStream x, long length)
			throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setBlob(int parameterIndex, Blob x) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setBlob(int parameterIndex, InputStream inputStream)
			throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setBlob(int parameterIndex, InputStream inputStream, long length)
			throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setByte(int parameterIndex, byte x) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setBytes(int parameterIndex, byte[] x) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setCharacterStream(int parameterIndex, Reader reader)
			throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setCharacterStream(int parameterIndex, Reader reader, int length)
			throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setCharacterStream(int parameterIndex, Reader reader,
			long length) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setClob(int parameterIndex, Clob x) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setClob(int parameterIndex, Reader reader) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setClob(int parameterIndex, Reader reader, long length)
			throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setDate(int parameterIndex, Date x) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setDate(int parameterIndex, Date x, Calendar cal)
			throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setFloat(int parameterIndex, float x) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setNCharacterStream(int parameterIndex, Reader value)
			throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setNCharacterStream(int parameterIndex, Reader value,
			long length) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setNClob(int parameterIndex, NClob value) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setNClob(int parameterIndex, Reader reader) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setNClob(int parameterIndex, Reader reader, long length)
			throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setNString(int parameterIndex, String value)
			throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setNull(int parameterIndex, int sqlType) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setNull(int parameterIndex, int sqlType, String typeName)
			throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setObject(int parameterIndex, Object x) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setObject(int parameterIndex, Object x, int targetSqlType)
			throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setObject(int parameterIndex, Object x, int targetSqlType,
			int scaleOrLength) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setRef(int parameterIndex, Ref x) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setRowId(int parameterIndex, RowId x) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setSQLXML(int parameterIndex, SQLXML xmlObject)
			throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setShort(int parameterIndex, short x) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}
	
	@Override
	public void setTime(int parameterIndex, Time x) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setTime(int parameterIndex, Time x, Calendar cal)
			throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setTimestamp(int parameterIndex, Timestamp x)
			throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setTimestamp(int parameterIndex, Timestamp x, Calendar cal)
			throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setURL(int parameterIndex, URL x) throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

	@Override
	public void setUnicodeStream(int parameterIndex, InputStream x, int length)
			throws SQLException {
		if (debug) throw new NoSuchMethodError("Method in PreparedStatement not Implemented");

	}

}
